@@ -27,6 +27,7 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+#include <canberra-gtk.h>
 
 #include "gvc-balance-bar.h"
 
@@ -45,6 +46,7 @@ struct GvcBalanceBarPrivate
         GtkAdjustment *adjustment;
         GtkSizeGroup  *size_group;
         gboolean       symmetric;
+        gboolean       click_lock;
 };
 
 enum
@@ -57,6 +59,16 @@ static void     gvc_balance_bar_class_init (GvcBalanceBarClass *klass);
 static void     gvc_balance_bar_init       (GvcBalanceBar      *balance_bar);
 static void     gvc_balance_bar_finalize   (GObject            *object);
 
+static gboolean on_scale_button_press_event   (GtkWidget      *widget,
+                                               GdkEventButton *event,
+                                               GvcBalanceBar  *bar);
+static gboolean on_scale_button_release_event (GtkWidget      *widget,
+                                               GdkEventButton *event,
+                                               GvcBalanceBar  *bar);
+static gboolean on_scale_scroll_event         (GtkWidget      *widget,
+                                               GdkEventScroll *event,
+                                               GvcBalanceBar  *bar);
+
 G_DEFINE_TYPE (GvcBalanceBar, gvc_balance_bar, GTK_TYPE_HBOX)
 
 static GtkWidget *
@@ -66,9 +78,7 @@ _scale_box_new (GvcBalanceBar *bar)
         GtkWidget            *box;
         GtkWidget            *sbox;
         GtkWidget            *ebox;
-        GtkWidget            *ivbox;
-        GtkWidget            *ihbox;
-        GtkWidget            *label;
+        GtkAdjustment        *adjustment = bar->priv->adjustment;
         char                 *str;
 
         bar->priv->scale_box = box = gtk_hbox_new (FALSE, 6);
@@ -86,27 +96,34 @@ _scale_box_new (GvcBalanceBar *bar)
 
         gtk_box_pack_start (GTK_BOX (sbox), priv->label, FALSE, FALSE, 0);
 
-        ivbox = gtk_vbox_new (FALSE, 0);
-        gtk_box_pack_start (GTK_BOX (box), ivbox, TRUE, TRUE, 0);
+        gtk_box_pack_start (GTK_BOX (sbox), priv->scale, TRUE, TRUE, 0);
 
-        gtk_box_pack_start (GTK_BOX (ivbox), priv->scale, TRUE, TRUE, 0);
-        ihbox = gtk_hbox_new (FALSE, 0);
-        gtk_box_pack_start (GTK_BOX (ivbox), ihbox, FALSE, FALSE, 0);
-        str = g_strdup_printf ("<small>%s</small>", _("Left"));
-        label = gtk_label_new (NULL);
-        gtk_label_set_markup (GTK_LABEL (label), str);
+        str = g_strdup_printf ("<small>%s</small>", C_("balance", "Left"));
+        gtk_scale_add_mark (GTK_SCALE (priv->scale), adjustment->lower , 
+                            GTK_POS_BOTTOM, str);
         g_free (str);
-        gtk_box_pack_start (GTK_BOX (ihbox), label, FALSE, FALSE, 0);
-        str = g_strdup_printf ("<small>%s</small>", _("Right"));
-        label = gtk_label_new (NULL);
-        gtk_label_set_markup (GTK_LABEL (label), str);
+
+        str = g_strdup_printf ("<small>%s</small>", C_("balance", "Right"));
+        gtk_scale_add_mark (GTK_SCALE (priv->scale),  adjustment->upper, 
+                            GTK_POS_BOTTOM, str);
         g_free (str);
-        gtk_box_pack_end (GTK_BOX (ihbox), label, FALSE, FALSE, 0);
+
+        gtk_scale_add_mark (GTK_SCALE (priv->scale), (adjustment->upper - adjustment->lower)/2 + adjustment->lower, 
+                            GTK_POS_BOTTOM, NULL);
 
         bar->priv->end_box = ebox = gtk_hbox_new (FALSE, 6);
         gtk_box_pack_start (GTK_BOX (box), ebox, FALSE, FALSE, 0);
 
-        gtk_range_set_update_policy (GTK_RANGE (priv->scale), GTK_UPDATE_DISCONTINUOUS);
+        gtk_range_set_update_policy (GTK_RANGE (priv->scale), GTK_UPDATE_CONTINUOUS);
+        ca_gtk_widget_disable_sounds (bar->priv->scale, FALSE);
+        gtk_widget_add_events (bar->priv->scale, GDK_SCROLL_MASK);
+
+        g_signal_connect (G_OBJECT (bar->priv->scale), "button-press-event",
+                          G_CALLBACK (on_scale_button_press_event), bar);
+        g_signal_connect (G_OBJECT (bar->priv->scale), "button-release-event",
+                          G_CALLBACK (on_scale_button_release_event), bar);
+        g_signal_connect (G_OBJECT (bar->priv->scale), "scroll-event",
+                          G_CALLBACK (on_scale_scroll_event), bar);
 
         if (bar->priv->size_group != NULL) {
                 gtk_size_group_add_widget (bar->priv->size_group, sbox);
@@ -254,6 +271,52 @@ on_right (pa_channel_position_t p)
         p == PA_CHANNEL_POSITION_TOP_REAR_RIGHT;
 }
 
+static gboolean
+on_scale_button_press_event (GtkWidget      *widget,
+                             GdkEventButton *event,
+                             GvcBalanceBar  *bar)
+{
+        bar->priv->click_lock = TRUE;
+
+        return FALSE;
+}
+
+static gboolean
+on_scale_button_release_event (GtkWidget      *widget,
+                               GdkEventButton *event,
+                               GvcBalanceBar  *bar)
+{
+        bar->priv->click_lock = FALSE;
+
+        return FALSE;
+}
+
+static gboolean
+on_scale_scroll_event (GtkWidget      *widget,
+                       GdkEventScroll *event,
+                       GvcBalanceBar  *bar)
+{
+        gdouble value;
+
+        value = gtk_adjustment_get_value (bar->priv->adjustment);
+
+        if (event->direction == GDK_SCROLL_UP) {
+                if (value + 0.01 > 1.0)
+                        value = 1.0;
+                else
+                        value = value + 0.01;
+                gtk_adjustment_set_value (bar->priv->adjustment, value);
+        } else if (event->direction == GDK_SCROLL_DOWN) {
+                if (value - 0.01 < 0)
+                        value = 0.0;
+                else
+                        value = value - 0.01;
+                gtk_adjustment_set_value (bar->priv->adjustment, value);
+        }
+
+        return TRUE;
+}
+
 static void
 on_adjustment_value_changed (GtkAdjustment *adjustment,
                              GvcBalanceBar *bar)
@@ -276,7 +339,6 @@ on_adjustment_value_changed (GtkAdjustment *adjustment,
         left_v = 1.0;
         right_v = 1.0;
 
-        /* FIXME: handle RTOL locales */
         if (val > 0) {
                 left_v = 1.0 - val;
         } else if (val < 0) {
@@ -320,7 +382,7 @@ gvc_balance_bar_init (GvcBalanceBar *bar)
                           G_CALLBACK (on_adjustment_value_changed),
                           bar);
 
-        bar->priv->label = gtk_label_new (_("Balance:"));
+        bar->priv->label = gtk_label_new_with_mnemonic (_("_Balance:"));
         gtk_misc_set_alignment (GTK_MISC (bar->priv->label),
                                 0.0,
                                 0.5);
@@ -333,6 +395,10 @@ gvc_balance_bar_init (GvcBalanceBar *bar)
         bar->priv->scale_box = _scale_box_new (bar);
         gtk_container_add (GTK_CONTAINER (frame), bar->priv->scale_box);
         gtk_widget_show_all (frame);
+
+        gtk_widget_set_direction (bar->priv->scale, GTK_TEXT_DIR_LTR);
+        gtk_label_set_mnemonic_widget (GTK_LABEL (bar->priv->label),
+                                       bar->priv->scale);
 }
 
 static void
