@@ -267,37 +267,7 @@ on_status_icon_scroll_event (GtkStatusIcon       *status_icon,
                              GdkEventScroll      *event,
                              GvcStreamStatusIcon *icon)
 {
-        GtkAdjustment *adj;
-
-        adj = GTK_ADJUSTMENT (gvc_channel_bar_get_adjustment (GVC_CHANNEL_BAR (icon->priv->bar)));
-
-        switch (event->direction) {
-        case GDK_SCROLL_UP:
-        case GDK_SCROLL_DOWN: {
-                gdouble volume;
-
-                volume = gtk_adjustment_get_value (adj);
-
-                if (event->direction == GDK_SCROLL_UP) {
-                        volume += adj->step_increment;
-                        if (volume > adj->upper) {
-                                volume = adj->upper;
-                        }
-                } else {
-                        volume -= adj->step_increment;
-                        if (volume < adj->lower) {
-                                volume = adj->lower;
-                        }
-                }
-
-                gtk_adjustment_set_value (adj, volume);
-                return TRUE;
-        }
-        default:
-                break;
-        }
-
-        return FALSE;
+        return gvc_channel_bar_scroll (GVC_CHANNEL_BAR (icon->priv->bar), event->direction);
 }
 
 static void
@@ -502,6 +472,15 @@ on_stream_is_muted_notify (GObject             *object,
         /* FIXME: update dock too */
 }
 
+static void
+on_stream_decibel_notify (GObject             *object,
+                          GParamSpec          *pspec,
+                          GvcStreamStatusIcon *icon)
+{
+        update_icon (icon);
+        /* FIXME: update dock too */
+}
+
 void
 gvc_stream_status_icon_set_display_name (GvcStreamStatusIcon *icon,
                                          const char          *name)
@@ -531,6 +510,9 @@ gvc_stream_status_icon_set_mixer_stream (GvcStreamStatusIcon *icon,
                 g_signal_handlers_disconnect_by_func (icon->priv->mixer_stream,
                                                       G_CALLBACK (on_stream_is_muted_notify),
                                                       icon);
+                g_signal_handlers_disconnect_by_func (icon->priv->mixer_stream,
+                                                      G_CALLBACK (on_stream_decibel_notify),
+                                                      icon);
                 g_object_unref (icon->priv->mixer_stream);
                 icon->priv->mixer_stream = NULL;
         }
@@ -554,6 +536,10 @@ gvc_stream_status_icon_set_mixer_stream (GvcStreamStatusIcon *icon,
                                   "notify::is-muted",
                                   G_CALLBACK (on_stream_is_muted_notify),
                                   icon);
+                g_signal_connect (icon->priv->mixer_stream,
+                                  "notify::decibel",
+                                  G_CALLBACK (on_stream_decibel_notify),
+                                  icon);
         }
 
         update_icon (icon);
