@@ -23,7 +23,8 @@
 #include "config.h"
 #endif
 
-#include <gnome.h>
+#include <string.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
 #include "element.h"
@@ -31,6 +32,11 @@
 #include "preferences.h"
 #include "track.h"
 #include "misc.h"
+#ifdef HAVE_SOUND_THEME
+#include "gvc-sound-theme-chooser.h"
+#endif
+
+G_DEFINE_TYPE (GnomeVolumeControlElement, gnome_volume_control_element, GTK_TYPE_NOTEBOOK)
 
 static void	gnome_volume_control_element_class_init	(GnomeVolumeControlElementClass *klass);
 static void	gnome_volume_control_element_init	(GnomeVolumeControlElement *el);
@@ -41,43 +47,12 @@ static void	cb_gconf			(GConfClient     *client,
 						 GConfEntry      *entry,
 						 gpointer         data);
 
-static GtkNotebookClass *parent_class = NULL;
-
-GType
-gnome_volume_control_element_get_type (void)
-{
-  static GType gnome_volume_control_element_type = 0;
-
-  if (!gnome_volume_control_element_type) {
-    static const GTypeInfo gnome_volume_control_element_info = {
-      sizeof (GnomeVolumeControlElementClass),
-      NULL,
-      NULL,
-      (GClassInitFunc) gnome_volume_control_element_class_init,
-      NULL,
-      NULL,
-      sizeof (GnomeVolumeControlElement),
-      0,
-      (GInstanceInitFunc) gnome_volume_control_element_init,
-      NULL
-    };
-
-    gnome_volume_control_element_type =
-	g_type_register_static (GTK_TYPE_NOTEBOOK, 
-				"GnomeVolumeControlElement",
-				&gnome_volume_control_element_info, 0);
-  }
-
-  return gnome_volume_control_element_type;
-}
 
 static void
 gnome_volume_control_element_class_init (GnomeVolumeControlElementClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
-  parent_class = g_type_class_ref (GTK_TYPE_NOTEBOOK);
-
   gobject_class->dispose = gnome_volume_control_element_dispose;
 }
 
@@ -86,30 +61,22 @@ gnome_volume_control_element_init (GnomeVolumeControlElement *el)
 {
   el->client = NULL;
   el->mixer = NULL;
-  el->appbar = NULL;
 }
 
 GtkWidget *
-gnome_volume_control_element_new (GstElement  *element,
-				  GConfClient *client,
-				  GnomeAppBar *appbar)
+gnome_volume_control_element_new (GConfClient  *client)
 {
   GnomeVolumeControlElement *el;
 
-  g_return_val_if_fail (GST_IS_MIXER (element), NULL);
-
   /* element */
   el = g_object_new (GNOME_VOLUME_CONTROL_TYPE_ELEMENT, NULL);
   el->client = g_object_ref (G_OBJECT (client));
-  el->appbar = appbar;
 
   gconf_client_add_dir (el->client, GNOME_VOLUME_CONTROL_KEY_DIR,
 			GCONF_CLIENT_PRELOAD_RECURSIVE, NULL);
   gconf_client_notify_add (el->client, GNOME_VOLUME_CONTROL_KEY_DIR,
 			   cb_gconf, el, NULL, NULL);
 
-  gnome_volume_control_element_change (el, element);
-
   return GTK_WIDGET (el);
 }
 
@@ -131,7 +98,7 @@ gnome_volume_control_element_dispose (GObject *object)
     el->mixer = NULL;
   }
 
-  G_OBJECT_CLASS (parent_class)->dispose (object);
+  G_OBJECT_CLASS (gnome_volume_control_element_parent_class)->dispose (object);
 }
 
 /*
@@ -139,12 +106,34 @@ gnome_volume_control_element_dispose (GObject *object)
  */
 
 gboolean
-gnome_volume_control_element_whitelist (GstMixerTrack *track,
-					gvc_whitelist *list)
+gnome_volume_control_element_whitelist (GstMixerTrack *track)
 {
   gint i, pos;
   gboolean found = FALSE;
 
+  /* Yes this is a hack. */
+  static struct {
+    gchar *label;
+    gboolean done;
+  } list[] = {
+
+/* Translator comment: the names below are a whitelist for which volume
+ * controls to show by default. Make sure that those match the translations
+ * of GStreamer-plugins' ALSA/OSS plugins. */
+    { "cd", FALSE },
+    { "line", FALSE },
+    { "mic", FALSE },
+    { "pcm", FALSE },
+    { "headphone", FALSE },
+    { "speaker", FALSE },
+    { "volume", FALSE },
+    { "master", FALSE },
+    { "digital output", FALSE },
+    { "recording", FALSE },
+    { "front", FALSE },
+    { NULL, FALSE }
+  };
+
   for (i = 0; !found && list[i].label != NULL; i++) {
     gchar *label_l = NULL;
 
@@ -174,7 +163,7 @@ gnome_volume_control_element_whitelist (GstMixerTrack *track,
  */
 
 static void
-update_tab_visibility (GnomeVolumeControlElement *el, gint page)
+update_tab_visibility (GnomeVolumeControlElement *el, gint page, gint tabnum)
 {
   const GList *item;
   gboolean visible = FALSE;
@@ -184,7 +173,7 @@ update_tab_visibility (GnomeVolumeControlElement *el, gint page)
        item != NULL; item = item->next) {
     GstMixerTrack *track = item->data;
     GnomeVolumeControlTrack *trkw =
-        g_object_get_data (G_OBJECT (track), "gnome-volume-control-trkw"); 
+        g_object_get_data (G_OBJECT (track), "gnome-volume-control-trkw");
 
     if (get_page_num (track) == page && trkw->visible) {
       visible = TRUE;
@@ -192,13 +181,47 @@ update_tab_visibility (GnomeVolumeControlElement *el, gint page)
     }
   }
 
-  t = gtk_notebook_get_nth_page (GTK_NOTEBOOK (el), page);
+  t = gtk_notebook_get_nth_page (GTK_NOTEBOOK (el), tabnum);
   if (visible)
     gtk_widget_show (t);
   else
     gtk_widget_hide (t);
 }
 
+static void
+cb_notify_message (GstBus *bus, GstMessage *message, gpointer data)
+{
+  GnomeVolumeControlElement *el = data;
+  GstMixerMessageType type;
+  GnomeVolumeControlTrack *trkw;
+  GstMixerTrack *track = NULL;
+  GstMixerOptions *options = NULL;
+
+  if (GST_MESSAGE_SRC (message) != GST_OBJECT (el->mixer)) {
+    /* not from our mixer - can't update anything anyway */
+    return;
+  }
+
+  /* This code only calls refresh if the first_track changes, because the
+   * refresh code only retrieves the current value from that track anyway */
+  type = gst_mixer_message_get_type (message);
+  if (type == GST_MIXER_MESSAGE_MUTE_TOGGLED) {
+    gst_mixer_message_parse_mute_toggled (message, &track, NULL);
+  } else if (type == GST_MIXER_MESSAGE_VOLUME_CHANGED) {
+    gst_mixer_message_parse_volume_changed (message, &track, NULL, NULL);
+  } else if (type == GST_MIXER_MESSAGE_OPTION_CHANGED) {
+    gst_mixer_message_parse_option_changed (message, &options, NULL);
+    track = GST_MIXER_TRACK (options);
+  } else {
+    return;
+  }
+
+  trkw = g_object_get_data (G_OBJECT (track),
+			    "gnome-volume-control-trkw");
+
+  gnome_volume_control_track_update (trkw);
+}
+
 /*
  * Change the element. Basically recreates this object internally.
  */
@@ -208,7 +231,6 @@ gnome_volume_control_element_change (GnomeVolumeControlElement *el,
 				     GstElement *element)
 {
   struct {
-    gchar *label;
     GtkWidget *page, *old_sep, *new_sep;
     gboolean use;
     gint pos, height, width;
@@ -217,25 +239,29 @@ gnome_volume_control_element_change (GnomeVolumeControlElement *el,
 						    GstMixer      *mixer,
 						    GstMixerTrack *track,
 						    GtkWidget     *left_sep,
-						    GtkWidget     *right_sep,
-						    GnomeAppBar   *appbar);
+						    GtkWidget     *right_sep);
   } content[4] = {
-    { _("Playback"), NULL, NULL, NULL, FALSE, 0, 5, 1,
+    { NULL, NULL, NULL, FALSE, 0, 5, 1,
       gnome_volume_control_track_add_playback },
-    { _("Recording"),  NULL, NULL, NULL, FALSE, 0, 5, 1,
+    { NULL, NULL, NULL, FALSE, 0, 5, 1,
       gnome_volume_control_track_add_recording },
-    { _("Switches"), NULL, NULL, NULL, FALSE, 0, 1, 3,
+    { NULL, NULL, NULL, FALSE, 0, 1, 3,
       gnome_volume_control_track_add_switch },
-    { _("Options"),  NULL, NULL, NULL, FALSE, 0, 1, 3,
+    { NULL, NULL, NULL, FALSE, 0, 1, 3,
       gnome_volume_control_track_add_option }
   };
-  gvc_whitelist list[] = whitelist_init_list;
-  gint i;
+  static gboolean theme_page = FALSE;
   const GList *item;
   GstMixer *mixer;
+  GstBus *bus;
+  gint i;
 
-  /* remove old pages */
-  while (gtk_notebook_get_n_pages (GTK_NOTEBOOK (el)) > 0) {
+  /* remove old pages, but not the "Sound Theme" page */
+  i = 0;
+  if (theme_page)
+    i = 1;
+
+  while (gtk_notebook_get_n_pages (GTK_NOTEBOOK (el)) > i) {
     gtk_notebook_remove_page (GTK_NOTEBOOK (el), 0);
   }
 
@@ -248,6 +274,7 @@ gnome_volume_control_element_change (GnomeVolumeControlElement *el,
 
       trkw = g_object_get_data (G_OBJECT (track),
 				"gnome-volume-control-trkw");
+      g_object_set_data (G_OBJECT (track), "gnome-volume-control-trkw", NULL);
       gnome_volume_control_track_free (trkw);
     }
   }
@@ -258,6 +285,15 @@ gnome_volume_control_element_change (GnomeVolumeControlElement *el,
   mixer = GST_MIXER (element);
   gst_object_replace ((GstObject **) &el->mixer, GST_OBJECT (element));
 
+  /* Bus for notifications */
+  if (GST_ELEMENT_BUS (mixer) == NULL) {
+    bus = gst_bus_new ();
+    gst_bus_add_signal_watch (bus);
+    g_signal_connect (G_OBJECT (bus), "message::element",
+		      (GCallback) cb_notify_message, el);
+    gst_element_set_bus (GST_ELEMENT (mixer), bus);
+  }
+
   /* content pages */
   for (i = 0; i < 4; i++) {
     content[i].page = gtk_table_new (content[i].width, content[i].height, FALSE);
@@ -271,7 +307,6 @@ gnome_volume_control_element_change (GnomeVolumeControlElement *el,
   for (item = gst_mixer_list_tracks (el->mixer);
        item != NULL; item = item->next) {
     GstMixerTrack *track = item->data;
-    GtkWidget *trackw;
     GnomeVolumeControlTrack *trkw;
     gchar *key;
     const GConfValue *value;
@@ -293,7 +328,7 @@ gnome_volume_control_element_change (GnomeVolumeControlElement *el,
     }
 
     /* visible? */
-    active = gnome_volume_control_element_whitelist (track, list);
+    active = gnome_volume_control_element_whitelist (track);
     key = get_gconf_key (el->mixer, track);
     if ((value = gconf_client_get (el->client, key, NULL)) != NULL &&
         value->type == GCONF_VALUE_BOOL) {
@@ -308,8 +343,7 @@ gnome_volume_control_element_change (GnomeVolumeControlElement *el,
     /* widget */
     trkw = content[i].get_track_widget (GTK_TABLE (content[i].page),
 					content[i].pos++, el->mixer, track,
-					content[i].old_sep, content[i].new_sep,
-					el->appbar);
+					content[i].old_sep, content[i].new_sep);
     gnome_volume_control_track_show (trkw, active);
 
     g_object_set_data (G_OBJECT (track),
@@ -330,11 +364,16 @@ gnome_volume_control_element_change (GnomeVolumeControlElement *el,
     }
 
     content[i].old_sep = content[i].new_sep;
-    content[i].use = TRUE;
+
+    if (active) {
+      content[i].use = TRUE;
+    }
   }
 
-  /* show */
-  for (i = 0; i < 4; i++) {
+  /* show - need to build the tabs backwards so that deleting the "Sound Theme"
+   * page can be avoided.
+   */
+  for (i = 3; i >= 0; i--) {
     GtkWidget *label, *view, *viewport;
     GtkAdjustment *hadjustment, *vadjustment;
 
@@ -358,14 +397,14 @@ gnome_volume_control_element_change (GnomeVolumeControlElement *el,
     gtk_container_add (GTK_CONTAINER (viewport), content[i].page);
     gtk_container_add (GTK_CONTAINER (view), viewport);
 
-    label = gtk_label_new (content[i].label);
-    gtk_notebook_append_page (GTK_NOTEBOOK (el), view, label);
+    label = gtk_label_new (get_page_description (i));
+    gtk_notebook_prepend_page (GTK_NOTEBOOK (el), view, label);
     gtk_widget_show (content[i].page);
     gtk_widget_show (viewport);
     gtk_widget_show (view);
     gtk_widget_show (label);
 
-    update_tab_visibility (el, i);
+    update_tab_visibility (el, i, 0);
   }
 
   /* refresh fix */
@@ -373,6 +412,42 @@ gnome_volume_control_element_change (GnomeVolumeControlElement *el,
        i >= 0; i--) {
     gtk_notebook_set_current_page (GTK_NOTEBOOK (el), i);
   }
+
+#ifdef HAVE_SOUND_THEME
+  /* Add tab for managing themes */
+  if (!theme_page) {
+    theme_page = TRUE;
+    GtkWidget *label, *view, *viewport, *sound_theme_chooser, *vbox;
+    GtkAdjustment *hadjustment, *vadjustment;
+
+    label = gtk_label_new (_("Sound Theme"));
+
+    view = gtk_scrolled_window_new (NULL, NULL);
+    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (view),
+				    GTK_POLICY_AUTOMATIC,
+				    GTK_POLICY_AUTOMATIC);
+
+    hadjustment = gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (view));
+    vadjustment = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (view));
+    viewport = gtk_viewport_new (hadjustment, vadjustment);
+    gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport), GTK_SHADOW_NONE);
+    gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport), GTK_SHADOW_NONE);
+
+    sound_theme_chooser = gvc_sound_theme_chooser_new ();
+    vbox = gtk_vbox_new (FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (vbox), sound_theme_chooser, TRUE, TRUE, 6);
+    gtk_container_add (GTK_CONTAINER (viewport), vbox);
+    gtk_container_add (GTK_CONTAINER (view), viewport);
+
+    gtk_widget_show_all (vbox);
+    gtk_widget_show (sound_theme_chooser);
+    gtk_widget_show (viewport);
+    gtk_widget_show (view);
+    gtk_widget_show (label);
+
+    gtk_notebook_append_page (GTK_NOTEBOOK (el), view, label);
+  }
+#endif
 }
 
 /*
@@ -399,7 +474,10 @@ cb_gconf (GConfClient *client,
 	g_object_get_data (G_OBJECT (track), "gnome-volume-control-trkw");
       gchar *key = get_gconf_key (el->mixer, track);
 
-      if (!strcmp (gconf_entry_get_key (entry), key)) {
+      g_return_if_fail (gconf_entry_get_key (entry) != NULL);
+      g_return_if_fail (key != NULL);
+
+      if (g_str_equal (gconf_entry_get_key (entry), key)) {
         GConfValue *value = gconf_entry_get_value (entry);
 
         if (value->type == GCONF_VALUE_BOOL) {
@@ -428,7 +506,7 @@ cb_gconf (GConfClient *client,
             if (trkw->visible && first[n])
               first[n] = FALSE;
           }
-          update_tab_visibility (el, page);
+          update_tab_visibility (el, page, page);
           break;
         }
       }
