@@ -216,6 +216,7 @@ main (int argc,
 	GOptionContext *goption_context;
 	GtkWidget *hbox;
 	GtkWidget *frame;
+	GnomeProgram *program;
 	vumeter *meter;
 	/*int time_id;*/
 	int i;
@@ -249,9 +250,9 @@ main (int argc,
 	goption_context = g_option_context_new(NULL);
 	g_option_context_add_main_entries(goption_context, options, GETTEXT_PACKAGE);
 
-	gnome_program_init ("vumeter", VERSION, LIBGNOMEUI_MODULE, argc, argv,
-				    GNOME_PARAM_GOPTION_CONTEXT, goption_context, 
-				    NULL);
+	program = gnome_program_init ("vumeter", VERSION, LIBGNOMEUI_MODULE, argc, argv,
+				      GNOME_PARAM_GOPTION_CONTEXT, goption_context, 
+				      NULL);
 #ifdef DEBUG
 	if (esd_host) {
 		g_print (_("Host is %s\n"), esd_host);
@@ -268,8 +269,8 @@ main (int argc,
 			  G_CALLBACK (gtk_main_quit), argv[0]);
 	
 	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-	gnome_window_icon_set_from_file (GTK_WINDOW (window),
-					 GNOME_ICONDIR"/gnome-vumeter.png");
+	gtk_window_set_icon_name (GTK_WINDOW (window),
+	                          (record ? "gnome-reclevel" : "gnome-vumeter"));
 	gtk_window_set_title (GTK_WINDOW (window),
 			      (record ? _("Recording level") : _("Volume Meter") ));
 	if (session_xpos >=0 && session_ypos >= 0) {
@@ -333,6 +334,8 @@ main (int argc,
 	/* time_id = gtk_timeout_add (50000, (GtkFunction)update_display, meter); */
 	
 	gtk_main ();
+
+	g_object_unref (program);
 	
 	return 0;
 }
